#include <vector>

#include "G35String.h"

#include <ESP8266WiFi.h>
#include <ESP8266httpUpdate.h>

const char* host = "holiday.home.flame.org";
const char* ssid = "home.flame.org";
const char* pass = "DietCokeLime!";
const uint16_t app_port = 4201;

const char *UPDATE_HOST = "multipass.home.flame.org";
const char *UPDATE_PATH = "/explorer/esp.bin";

WiFiServer server(app_port);

const char *version = "0.1.23";

std::vector<G35String *>lights = {
  new G35String(12, 25),
  new G35String(15, 36),
  new G35String(14, 36),
  new G35String(13, 36),
  new G35String(4, 36),
  new G35String(5, 36),
};

void setupWifi()
{
  Serial.printf("Version: %s\n", version);
  Serial.printf("Sketch size: %u\n", ESP.getSketchSize());
  Serial.printf("Free size: %u\n", ESP.getFreeSketchSpace());

  WiFi.mode(WIFI_STA);
  WiFi.begin(ssid, pass);
  if (WiFi.waitForConnectResult() == WL_CONNECTED) {
    Serial.print("IP address: ");
    Serial.println(WiFi.localIP());
  }

  server.begin();
}

void setup()
{
  Serial.begin(115200);
  delay(100);

  //Serial.setDebugOutput(true);
 
  setupWifi();

  for (auto light : lights) {
    light->enumerate();
    light->broadcast_intensity(0);
  }
}

void doUpdate()
{
  auto ret = ESPhttpUpdate.update(UPDATE_HOST, 80, UPDATE_PATH, version);
  switch (ret) {
    case HTTP_UPDATE_FAILED:
      Serial.println("[update] Update failed.");
      Update.printError(Serial);
      break;
    case HTTP_UPDATE_NO_UPDATES:
      Serial.println("[update] Update no Update.");
      break;
    case HTTP_UPDATE_OK:
      Serial.println("[update] Update ok."); // may not called we reboot the ESP
      break;
  }
}

WiFiClient client;
uint8_t stringId;

void wifiLoop()
{
  if (!client.connected()) {
    if (!server.hasClient())
      return;
    client = server.available();
  }
  if (!client.connected()) {
    client.stop();
    return;
  }

  int command = client.read();
  if (command == -1)
    return;

  if (command == 'S') {
    stringId = client.read();
  } else if (command == 'L') {
    uint8_t buffer[4];
    int readlen = client.readBytes(buffer, sizeof buffer);
    if (readlen != sizeof(buffer)) {
      Serial.printf("Only read %d bytes\n", readlen);
      return;
    }
    uint8_t lightId = buffer[0];
    uint8_t brightness = buffer[1];
    uint8_t rg = buffer[2];
    uint8_t b = buffer[3];

    if (brightness > G35::MAX_INTENSITY)
      brightness = G35::MAX_INTENSITY;

    color_t color = COLOR((rg & 0xf0) >> 4, (rg & 0x0f), b & 0x0f);

    auto string = lights[stringId];
    if (string == nullptr)
      Serial.println("Invalid string ID");
    else
      lights[stringId]->set_color(lightId, brightness, color);
  } else if (command == 'R') {
    client.stop();
    delay(1000);
    ESP.reset();
  } else if (command == 'O') {
    Serial.println("Attempting firmware upgrade");
    client.stop();
    delay(1000);
    doUpdate();
  }
}

void loop()
{
  wifiLoop();
}

